@@ -13,13 +13,20 @@
 # Imports
 #-----------------------------------------------------------------------------
 
+import os
 from .select import public_api, select_backend
 
-
-try:
-    _ns = select_backend('zmq.backend.cython')
-except ImportError:
-    _ns = select_backend('zmq.backend.cffi')
+if 'PYZMQ_BACKEND' in os.environ:
+    backend = os.environ['PYZMQ_BACKEND']
+    if backend in ('cython', 'cffi'):
+        backend = 'zmq.backend.%s' % backend
+    _ns = select_backend(backend)
+else:
+    # default to cython, fallback to cffi
+    try:
+        _ns = select_backend('zmq.backend.cython')
+    except ImportError:
+        _ns = select_backend('zmq.backend.cffi')
 
 globals().update(_ns)
 
