@@ -74,16 +74,27 @@ class Socket(object):
     _zmq_socket = None
     _closed = None
     _ref = None
+    _shadow = False
 
-    def __init__(self, context, sock_type):
+    def __init__(self, context=None, socket_type=None, shadow=None):
         self.context = context
-        self.socket_type = sock_type
-        self._zmq_socket = C.zmq_socket(context._zmq_ctx, sock_type)
+        if shadow is not None:
+            self._zmq_socket = ffi.cast("void *", shadow)
+            self._shadow = True
+        else:
+            self._shadow = False
+            self._zmq_socket = C.zmq_socket(context._zmq_ctx, socket_type)
         if self._zmq_socket == ffi.NULL:
             raise ZMQError()
         self._closed = False
-        self._ref = self.context._add_socket(self)
-
+        if context:
+            self._ref = context._add_socket(self)
+    
+    @property
+    def underlying(self):
+        """The address of the underlying libzmq socket"""
+        return int(ffi.cast('size_t', self._zmq_socket))
+    
     @property
     def closed(self):
         return self._closed
@@ -94,13 +105,17 @@ class Socket(object):
             if self._zmq_socket is not None:
                 rc = C.zmq_close(self._zmq_socket)
             self._closed = True
-            self.context._rm_socket(self._ref)
+            if self.context:
+                self.context._rm_socket(self._ref)
         return rc
 
     def __del__(self):
-        self.close()
+        if not self._shadow:
+            self.close()
 
     def bind(self, address):
+        if isinstance(address, unicode):
+            address = address.encode('utf8')
         rc = C.zmq_bind(self._zmq_socket, address)
         if rc < 0:
             if IPC_PATH_MAX_LEN and C.zmq_errno() == errno_mod.ENAMETOOLONG:
@@ -116,14 +131,20 @@ class Socket(object):
                 _check_rc(rc)
 
     def unbind(self, address):
+        if isinstance(address, unicode):
+            address = address.encode('utf8')
         rc = C.zmq_unbind(self._zmq_socket, address)
         _check_rc(rc)
 
     def connect(self, address):
+        if isinstance(address, unicode):
+            address = address.encode('utf8')
         rc = C.zmq_connect(self._zmq_socket, address)
         _check_rc(rc)
 
     def disconnect(self, address):
+        if isinstance(address, unicode):
+            address = address.encode('utf8')
         rc = C.zmq_disconnect(self._zmq_socket, address)
         _check_rc(rc)
 
@@ -137,31 +158,31 @@ class Socket(object):
                 raise TypeError("not a bytes sockopt: %s" % option)
             length = len(value)
         
-        low_level_data = initialize_opt_pointer(option, value, length)
+        c_data = initialize_opt_pointer(option, value, length)
 
-        low_level_value_pointer = low_level_data[0]
-        low_level_sizet = low_level_data[1]
+        c_value_pointer = c_data[0]
+        c_sizet = c_data[1]
 
         rc = C.zmq_setsockopt(self._zmq_socket,
                                option,
-                               ffi.cast('void*', low_level_value_pointer),
-                               low_level_sizet)
+                               ffi.cast('void*', c_value_pointer),
+                               c_sizet)
         _check_rc(rc)
 
     def get(self, option):
-        low_level_data = new_pointer_from_opt(option, length=255)
+        c_data = new_pointer_from_opt(option, length=255)
 
-        low_level_value_pointer = low_level_data[0]
-        low_level_sizet_pointer = low_level_data[1]
+        c_value_pointer = c_data[0]
+        c_sizet_pointer = c_data[1]
 
         rc = C.zmq_getsockopt(self._zmq_socket,
                                option,
-                               low_level_value_pointer,
-                               low_level_sizet_pointer)
+                               c_value_pointer,
+                               c_sizet_pointer)
         _check_rc(rc)
         
-        sz = low_level_sizet_pointer[0]
-        v = value_from_opt_pointer(option, low_level_value_pointer, sz)
+        sz = c_sizet_pointer[0]
+        v = value_from_opt_pointer(option, c_value_pointer, sz)
         if option != zmq.IDENTITY and option in zmq.constants.bytes_sockopts and v.endswith(b'\0'):
             v = v[:-1]
         return v
